import os

from django.db import models


def full_url(url='', domain=None, protocol='https'):
    """
    Prepend protocol (default to https) and domain name (default from the Site framework) to an url
    """
    if domain is None:
        from django.contrib.sites.models import Site
        domain = Site.objects.get_current().domain
    return f'{protocol}://{domain}{url}'


def enum_to_choices(enum):
    return ((item.value, item.name) for item in list(enum))


def query_sum(queryset, field):
    """
    Let the DBMS perform a sum on a queryset
    """
    return queryset.aggregate(s=models.functions.Coalesce(models.Sum(field), 0))['s']


def get_env(env_file='.env'):
    """
    Set default environment variables from .env file
    """
    try:
        with open(env_file) as f:
            for line in f.readlines():
                try:
                    key, val = line.split('=', maxsplit=1)
                    os.environ.setdefault(key.strip(), val.strip())
                except ValueError:
                    pass
    except FileNotFoundError:
        pass



from django.db import models
from django.urls import reverse
from django.utils.safestring import mark_safe

from autoslug import AutoSlugField

from .utils import full_url
from .querysets import NameOrderedQuerySet


class Links(object):
    absolute_url_detail = True

    def get_absolute_url(self):
        app, model = self._meta.app_label, self._meta.model_name
        if self.absolute_url_detail:
            return reverse(f'{app}:{model}', kwargs={'slug': self.slug} if hasattr(self, 'slug') else {'pk': self.pk})
        else:
            return reverse(f'{app}:{model}s')

    def get_full_url(self):
        return full_url(self.get_absolute_url())

    def get_admin_url(self):
        return reverse(f'admin:{self._meta.app_label}_{self._meta.model_name}_change', args=[self.id])

    def get_link(self):
        return mark_safe(f'<a href="{self.get_absolute_url()}">{self}</a>')


class TimeStampedModel(models.Model):
    created = models.DateTimeField(auto_now_add=True)
    updated = models.DateTimeField(auto_now=True)

    class Meta:
        abstract = True


class NamedModel(models.Model):
    name = models.CharField(max_length=200, unique=True)
    slug = AutoSlugField(populate_from='name', unique=True)

    objects = NameOrderedQuerySet.as_manager()

    class Meta:
        abstract = True

    def __str__(self):
        return self.name



#!/usr/bin/env python

import os
import re
from subprocess import check_output

from setuptools import setup

with open(os.path.join(os.path.dirname(__file__), 'README.md')) as readme:
    README = readme.read()

with open(os.path.join(os.path.dirname(__file__), 'Pipfile')) as pipfile:
    content = pipfile.read()
    REQUIREMENTS = re.findall('''\\n *['"]?([\w-]*)['"]? *=''', content.split('packages]')[1])
    PYTHON_VERSION = re.search('''python_version *= *['"]*([\d.]+)['"]?''', content)[1]

VERSION = [tag for tag in check_output(['git', 'tag', '-l']).decode().split() if tag.startswith('v')][-1][1:]

# allow setup.py to be run from any path
os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))

setup(
    name='ndh',
    version=VERSION,
    packages=['ndh'],
    install_requires=REQUIREMENTS,
    include_package_data=True,
    license='BSD',
    description='Nim’s Django Helpers',
    long_description=README,
    long_description_content_type="text/markdown",
    url='https://github.com/nim65s/ndh',
    author='Guilhem Saurel',
    author_email='webmaster@saurel.me',
    python_requires=f'>={PYTHON_VERSION}',
    classifiers=[
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
    ],
)



from django.contrib.auth.mixins import UserPassesTestMixin


class SuperUserRequiredMixin(UserPassesTestMixin):
    def test_func(self):
        return self.request.user.is_superuser


class NDHFormMixin(object):
    template_name = 'ndh/base_form.html'
    title = ''

    def get_context_data(self, **kwargs):
        return super().get_context_data(title=self.title, **kwargs)


class NDHDeleteMixin(NDHFormMixin):
    template_name = 'ndh/base_delete.html'



from django.conf import settings


def settings_constants(request):
    if hasattr(settings, 'NDH_TEMPLATES_SETTINGS'):
        return {key: getattr(settings, key) for key in settings.NDH_TEMPLATES_SETTINGS}
    return {}
